%% Simulation of Markov chains for different weather scenarios
% Using the already created Markov chain transistion matrix to simulate the 
% weather and plotting different figures to see how different weather
% sceanarios may affect the operability and planning of an operation. 

clear all;
clc;

N=1000;    % Number of simulations 
n=2160;     % hours    n=2688 for HigSeason and n=2160 for Winter
states=10; 
MaxHs=4;    % Will differ for different weather data. Winter=4, HigSeason=5
            % Depends on the max wave height the service vessel can work at
            
sampleMatrix=zeros(N,n);
meanTimeMatrix=zeros(N,states);
stateRange=0.3477;         %Winter=0.3477   
                           %High Season= 0.2852 
                           % Highest wave height from the data collected.

%% Trabsistion matrix for the two selected weather scenarios
% (The transitions matrices are found using an other script.) 
% Choose the weather scenario in which you want to simulate

% Winter Scenario. 
P= [0.3333    0.5833         0         0    0.0833         0         0         0         0         0;
    0.0165    0.7783    0.1981    0.0071         0         0         0         0         0         0;
         0    0.1403    0.7063    0.1518    0.0017         0         0         0         0         0;
         0    0.0051    0.2392    0.6107    0.1374    0.0076         0         0         0         0;
    0.0035         0    0.0035    0.1938    0.6263    0.1661    0.0069         0         0         0;
         0         0         0    0.0054    0.2703    0.5459    0.1784         0         0         0;
         0         0         0         0    0.0161    0.2661    0.5645    0.1532         0         0;
         0         0         0         0         0         0    0.2184    0.6207    0.1609         0;
         0         0         0         0         0         0         0    0.4242    0.4242    0.1515;
         0         0         0         0         0         0         0         0    0.8333    0.1667];
%  

% % High Season sceanrio
% P= [0.9149    0.0831    0.0010    0.0010         0         0         0         0         0         0;
%     0.0859    0.8251    0.0848    0.0031    0.0010         0         0         0         0         0;
%          0    0.1824    0.7021    0.1132    0.0023         0         0         0         0         0;
%     0.0054    0.0269    0.2527    0.5914    0.1183    0.0054         0         0         0         0;
%          0    0.0154    0.0154    0.3538    0.4615    0.1538         0         0         0         0;
%          0         0         0         0    0.2683    0.5122    0.2195         0         0         0;
%          0         0         0         0         0    0.5000    0.4444    0.0556         0         0;
%          0         0         0         0         0         0    0.1250    0.7500    0.1250         0;
%          0         0         0         0         0         0         0    0.3333    0.3333    0.3333;
%          0         0         0         0         0         0         0         0    0.3333    0.6667];

%% Running N simulations, and picking a random nunber using monte carlo sim. 
    
for k=1:N
    simMC=zeros(1,n); 
    simMC(1)=round(rand(1)*(states -1)); % Generating a random start state
                                         % for each simulation.
    
        for i=2:n 

        simMC(i)=randsample(states,1,true,P(simMC(i-1)+1,:))-1 ;
        end

        sampleMatrix(k,:)=simMC;
    
        for j=1:states 
    
        meanTimeMatrix(k,j)=sum(simMC(:)==j - 1 ) ;
    
        end

   
end

%% Using the simulated weather to find what we want to investigate

% Probability of each simulation to be in a given state
p_pr_sim=meanTimeMatrix/n; 

 
%Finding mean probability of time spent in each state (meanProb) and 
% mean probability of being under a certain state (meanProbunder). 
meanProb=zeros(1,states);
meanProbunder=zeros(1,states);
for i =1:states 
    meanProb(i)=mean(meanTimeMatrix(:,i))/n;
    
    meanProbunder(i)=sum(meanProb(1:i));
end



% %% Probablity that the waves will be under a the wave heights selected to
%    be in the intervals.
Wave_prob=zeros(states,2);

for w = 1:states 
    Wave_prob(w,1)= stateRange*w;
    Wave_prob(w,2)= meanProbunder(w);

end

%% Find out how many hours the bad weather will last if maximum wave height
%  is the selected MaxHs.

Cray=zeros(N,n);

for k= 1:N
    for p= 1:n
        
       if sampleMatrix(k,p)>=MaxHs
           Cray(k,p)=1;
       else 
           Cray(k,p)= 0;
       end
    end
end    

Beslut(:,1)=Cray(:,1);


 for q=1:N  
    count=1;
    for r=2:n

      if  Cray(q,r)==1 && Cray(q,r-1)==1 
           Beslut(q,count)= Beslut(q,count)+1;    % Generating a matrix that counts how many hours
                                                  % in a row the weather is worse than MaxHs                                            
      elseif Cray(q,r)==1 && Cray(q,r-1)==0
          count= count+1;
          Beslut(q,count)=1;
       
      end
    
    end
    
    
    if Beslut(q,1)==0
       Beslut(q,:)=circshift(Beslut(q,:), [0,-1]);  % Shifts the row that is starting with zero one to the left
    end
    
    
    
 end
 
% Find the number of times each simulation has bad weather. 
Number=zeros(1,N);
for t=1:N
         
     for s=1:length(Beslut(1,:))
         
         if Beslut(t,s)==0
             Number(t)=length(Beslut(t,1:s-1));    % Finding the length of each vector to find mean
             break 
         end 
         
     end
   
    if Beslut(t,length(Beslut(1,:)))>=1
        Number(t)=length(Beslut(1,:));
    end
end




% Count the number of hours the bad weather lasts and group thenm in
% selected intervals.
 
[m, n]=size(Beslut);
count1=ones(1,7);
for i=1:m
    
    for j=1:n
   
        if 0 < Beslut(i,j)  && Beslut(i,j)<= 2
         Inter_W1(count1(1),1)= Beslut(i,j);
         count1(1)= count1(1) +1;
         
        elseif 2 < Beslut(i,j)  && Beslut(i,j)<= 4
            Inter_W2(count1(2),1)= Beslut(i,j);
            count1(2)= count1(2) +1;

        elseif  4 < Beslut(i,j)  && Beslut(i,j)<= 8
            Inter_W3(count1(3),1)= Beslut(i,j);
            count1(3)= count1(3) +1;

        elseif  8 < Beslut(i,j)  && Beslut(i,j)<= 12
            Inter_W4(count1(4),1)= Beslut(i,j);
            count1(4)= count1(4) +1;

        elseif  12 < Beslut(i,j)  && Beslut(i,j)<= 24
            Inter_W5(count1(5),1)= Beslut(i,j);
            count1(5)= count1(5) +1;

        elseif  24 < Beslut(i,j)  && Beslut(i,j)<= 48
           Inter_W6(count1(6),1)= Beslut(i,j);
            count1(6)= count1(6) +1;

        elseif  48 < Beslut(i,j)  
            Inter_W7(count1(7),1)= Beslut(i,j);
            count1(7)= count1(7) +1;
        end
    end
end


%% Name the intervals so that it can be seen in the plot
%G=[Inter_W1 ;Inter_W2; Inter_W3; Inter_W4; Inter_W5; Inter_W6 ; Inter_W7];

Group=[repmat({'1-12 h'},length(Inter_W1)+ length(Inter_W2) + length(Inter_W3)+ length(Inter_W4),1);...
    repmat({'13-24 h'},length(Inter_W5),1); repmat({'25-48 h'},length(Inter_W6),1);...
       repmat({'49-max h'},length(Inter_W7),1)];
      
% Find the length of all the 
Gr=[Inter_W1 ;Inter_W2; Inter_W3; Inter_W4; Inter_W5; Inter_W6 ; Inter_W7];



% Find the probability of each chosen interval to compare how long the bad 
% weather with last for 
L=length(Gr);
Pros=[length(Inter_W1)/L length(Inter_W2)/L length(Inter_W3)/L ...
    length(Inter_W4)/L length(Inter_W5)/L length(Inter_W6)/L ...
    length(Inter_W7)/L];



%% Best fit function to smooth the plot on operability

% For high season
 Interval=Wave_prob(:,1);
 pop=Wave_prob(:,2);
 population = fit( Interval, pop, 'poly6');  % Use poly2 to poly6 to see what fits best
                                             % poly6 for fits best for both 
                                             % scenarios
                                             
%% Making a vector with the states from 0 to 9 

Vector=zeros(1,states);

for u=1:10
    Vector(u)=u-1;   
end

%% Plots

%% Plot of the random markov chain simulation

% Shows the operability with limits for the selected service vessels
figure(1)
a1=plot( population,'b');  
hold on
% plot(Wave_prob(:,1), Wave_prob(:,2),'-', 'Linewidth', 2) % Real plot from
                                                           % simulation
hold on
plot([1.2 1.2],[0 1],'k - ','Linewidth', 2);
hold on
plot([1.4 1.4],[0 1],'r-' ,'Linewidth', 2);
axis([0 inf 0.2 1.0]);
set(a1,'linewidth',2);
axis([0 inf 0.2 1]);
title(' Operability of maintenance vessels');
legend('Real wave height data','Limit: cat 15',...
    'Limit: cat 25, mono 25, mono40','Location', 'Northwest');
set(gca,'FontSize',30)
ylim([meanProb(1,1), inf]);
xlim([Wave_prob(1,1)-0.05, 1.6]);
xlabel('Significant wave height [m]');
ylabel('Probability');
set(gca,'xtick',0:0.2:1.6)
grid on;


% Shows the mean probability of time spent for each state
figure(2)
th=boxplot(p_pr_sim);
title('Probability of being in a specific state');
set(gca,'XTickLabel',Vector(1,:))
xlabel('State');
ylabel('Probability');
set(th,'linewidth',1.5);
set(gca,'FontSize',30)
grid on;

% Shows the mean duration of bad weather for given intervals
figure(3)
bh=boxplot(Gr,Group);
title('Mean duration of bad weather for given intervals');
xlabel('Time intervals');
set(bh,'linewidth',2);
set(gca,'FontSize',30)
ylabel('Hours');
grid on;

% Shows probability for duration of bad weather
figure(4)
bar(1:length(Pros),Pros);
set(gca,'XTickLabel',{'1-2', '3-4', '5-8', '9-12', '12-24','25-48','49-Max'})
title('Probability for duration of bad weather ');
xlabel('Hours');
ylabel('Probability');
grid on
set(gca,'FontSize',30)
ylim([0, max(Pros)+ 0.05 ]);


% figure(5)
% plot(1:10,p_pr_sim,'-o'); % Showing the of probability for each state  





%% 
clearvars i k n N states P Steps step count w stateRange j p q r t i j k m...
    p r s  g  Cray bh a1 Beslut gh Beslut L pop MaxHS MaxVal count1 Gr ...
    u Group population simMC th