%% Plotting the real weather data obtained  to compare 
%  with Markov chain simultation

clear all;
interval=10; % 10 intervals are chosen due to absorbing states if we had 
             % chosen to increase number of states

             
             
% Importing the weather data recieved from Sintef Ocean.              
Winter = xlsread('Hs data Marine Harvest and Salmar.xlsx','Sheet1','D7453:D9612');
Spring = xlsread('Hs data Marine Harvest and Salmar.xlsx','Sheet1','D853:D3060');


% These weather scenarios are not plotted, but are included if one want 
% look at them at a later stage.

Summer = xlsread('Hs data Marine Harvest and Salmar.xlsx','Sheet1','D3061:D5268');
Autumn = xlsread('Hs data Marine Harvest and Salmar.xlsx','Sheet1','D5269:D7452');
HighSeason = xlsread('Hs data Marine Harvest and Salmar.xlsx','Sheet1','D3685:D6372');
Year = xlsread('Hs data Marine Harvest and Salmar.xlsx','Sheet1','D152:D8940');

% Spring     = 853  ->  3060    %In the Excel file receivedm, these are 
% Summer     = 3061 ->  5268    % the number of rows that should be 
% Autumn     = 5269 ->  7452    % imported. 
% Winter     = 7453 ->  9612
% Week 24-41 = 3685 ->  6372


% Finding max wave state of each season
Max_Wi=max(Winter);
Max_His=max(HighSeason);

Max_Sp=max(Spring);
Max_Sum=max(Summer);
Max_Au=max(Autumn);
Max_Y= max(Year);

% Creating the zero vectors for all weather scenarios
int_Spring=zeros(1,interval);
int_Summer=zeros(1,interval);
int_Autumn=zeros(1,interval);
int_Winter=zeros(1,interval);
int_HighSeason=zeros(1,interval);
int_Year=zeros(1,interval);


% Finding the wave heights for each state
for b=1:interval
    
    int_Spring(b)= (Max_Sp/interval)*b;
    int_Summer(b)=(Max_Sum/interval)*b;
    int_Autumn(b)=(Max_Au/interval)*b;
    int_Winter(b)=(Max_Wi/interval)*b;
    int_HighSeason(b)=(Max_His/interval)*b;
    int_Year(b)=(Max_Y/interval)*b;

end


% Sorting the vector in chronical order from lowest wave height to 
% maximum. 
Chron_Spring=sort(Spring);
Chron_Summer=sort(Summer);
Chron_Autumn=sort(Autumn);
Chron_Winter=sort(Winter);
Chron_HighSeason=sort(HighSeason);
Chron_Year=sort(Year);



% Count the number of data points in each state. State 2 will have both the
% number of data point from state 1 and 2. Stat 3, both 1, 2 and 3. 
% And so on. 

Prob=zeros(6,interval);
count=zeros(6,1);

for i=1:interval
    
     count(1)=0;
    for j=1:length(Spring)
        if Chron_Spring(j)<=int_Spring(i)
            count(1)=count(1) +1;
            Prob(1,i)=count(1);
          
        end
    end
    
    count(2)=0;
    for j=1:length(Summer)
        if Chron_Summer(j)<=int_Summer(i)
            count(2)=count(2) +1;
            Prob(2,i)=count(2);
        end
    end
    count(3)=0;
    for j=1:length(Autumn)
        if Chron_Autumn(j)<=int_Autumn(i)
            count(3)=count(3) +1;
            Prob(3,i)=count(3);
        end
    end
    count(4)=0;
    for j=1:length(Winter)
        if Chron_Winter(j)<=int_Winter(i)
            count(4)=count(4) +1;
            Prob(4,i)=count(4);
        end
    end
    count(5)=0;
    for j=1:length(HighSeason)
        if Chron_HighSeason(j)<=int_HighSeason(i)
            count(5)=count(5) +1;
            Prob(5,i)=count(5); 
        end
    end
    count(6)=0;
    for j=1:length(Year)
        if Chron_Year(j)<=int_Year(i)
            count(6)=count(6) +1;
            Prob(6,i)=count(6);
        end
    end   
            
end

Prob(1,interval)=Prob(1,interval)+1; % For some reason it won't count one 
                                     % of the 2 last data points for state 
                                     % 10 during the spring season. 
                                     % So have added it to make the
                                     % probability correct
 
% Count how many data points that are for each state by subtracting the 
% number point that already are in the previous states.                                       
for r=2:interval
    
  Prob(1,r)=Prob(1,r)-sum(Prob(1,1:r-1));
  Prob(2,r)=Prob(2,r)-sum(Prob(2,1:r-1));
  Prob(3,r)=Prob(3,r)-sum(Prob(3,1:r-1));
  Prob(4,r)=Prob(4,r)-sum(Prob(4,1:r-1));
  Prob(5,r)=Prob(5,r)-sum(Prob(5,1:r-1));
  Prob(6,r)=Prob(6,r)-sum(Prob(6,1:r-1));
    
end 

% Find the probability of time spent in each state.
Prob(1,:)=Prob(1,:)/length(Spring);
Prob(2,:)=Prob(2,:)/length(Summer);
Prob(3,:)=Prob(3,:)/length(Autumn);
Prob(4,:)=Prob(4,:)/length(Winter);
Prob(5,:)=Prob(5,:)/length(HighSeason);
Prob(6,:)=Prob(6,:)/length(Year);





% Add the probability of the previous states to find the operability of
% being able perform maintenance at this state.

meanProbunder=zeros(6,interval);
for p =1: interval
   
    meanProbunder(1,p)=sum(Prob(1,1:p));
    meanProbunder(2,p)=sum(Prob(2,1:p));
    meanProbunder(3,p)=sum(Prob(3,1:p));
    meanProbunder(4,p)=sum(Prob(4,1:p));
    meanProbunder(5,p)=sum(Prob(5,1:p));
    meanProbunder(6,p)=sum(Prob(6,1:p));
    
end

%% Best fit function to smooth the plot

% For high season scenario
 IntervalHighS=transpose(int_HighSeason);
 popHighS=transpose(meanProbunder(5,:));
 populationHighS = fit( IntervalHighS, popHighS, 'poly6');

% For Winter scenario
 IntervalWinter=transpose(int_Winter);
 popWinter=transpose(meanProbunder(4,:));
 populationWinter = fit( IntervalWinter, popWinter, 'poly6');
%% Plots 


figure(1)
a1=plot( populationHighS,'b');  
hold on
%plot(transpose(int_HighSeason),transpose(meanProbunder(5,:))...
%,'-', 'Linewidth', 2);         % Real plot: to see what type of polynomial
                                % that best fit with the curve. 
hold on
plot([1.2 1.2],[0 1],'k - ','Linewidth', 2);
hold on
plot([1.4 1.4],[0 1],'r-' ,'Linewidth', 2);
axis([0 inf 0.2 1.0]);
set(a1,'linewidth',2);
grid on
axis([0 inf 0.2 1]);
title('HIGH SEASON: Operability of maintenance vessels');
legend('Real wave height data','Limit: cat 15',...
    'Limit: cat 25, mono 25...,mono40','Location', 'Northwest');
set(gca,'FontSize',30)
ylim([meanProbunder(5,1), inf]);
xlim([IntervalHighS(1,1)-0.05, 1.6]);
xlabel('Significant wave height [m]');
ylabel('Probability');
set(gca,'xtick',[0:0.2:1.6])
grid on;


figure(2)
a1=plot( populationWinter,'b');  
hold on
%plot(transpose(int_HighSeason),transpose(meanProbunder(4,:)) ,'-', 'Linewidth', 2)
hold on
plot([1.2 1.2],[0 1],'k - ','Linewidth', 2);
hold on
plot([1.4 1.4],[0 1],'r-' ,'Linewidth', 2);
axis([0 inf 0.2 1.0]);
set(a1,'linewidth',2);
grid on
axis([0 inf 0.2 1]);
title('WINTER: Operability of maintenance vessels');
legend('Real wave height data','Limit: cat 15','Limit: cat 25, mono 25, mono40','Location', 'Northwest');
set(gca,'FontSize',30)
ylim([meanProbunder(4,1), inf]);
xlim([IntervalWinter(1,1)-0.05, 1.6]);
xlabel('Significant wave height [m]');
ylabel('Probability');
set(gca,'xtick',[0:0.2:1.6])
grid on;

%figure(1)
% plot(transpose(int_Winter),transpose(meanProbunder(4,:)) ,'-', 'Linewidth', 2)
% hold on
% plot([1.2 1.2],[0 1],'k - ','Linewidth', 1)
% hold on
% plot([1.4 1.4],[0 1],'r-' ,'Linewidth', 1)
% grid on
% axis([0 inf 0.2 1]);
% title(' WINTER: Operability of maintenance vessels');
% legend('Real wave height data','Limit: cat 15','Limit: cat 25, mono 25, mono40','Location', 'east')
% ylim([meanProbunder(4,1), inf]);
% xlabel('Significant wave height [m]');
% ylabel('Probability');

% figure(2)
% plot(transpose(int_HighSeason),transpose(meanProbunder(5,:)) ,'-', 'Linewidth', 2)
% hold on
% plot([1.2 1.2],[0 1],'k - ','Linewidth', 1)
% hold on
% plot([1.4 1.4],[0 1],'r-' ,'Linewidth', 1)
% grid on
% axis([0 inf 0.2 1]);
% title('HIGH SEASON: Operability of maintenance vessels');
% legend('Real wave height data','Limit: cat 15','Limit: cat 25, mono 25, mono40','Location', 'east')
% ylim([meanProbunder(5,1), inf]);
% xlabel('Significant wave height [m]');
% ylabel('Probability');

% figure(3)
% plot(transpose(int_Spring),transpose(meanProbunder(1,:)),'-', 'Linewidth', 2)
% hold on
% plot([1.2 1.2],[0 1],'k - ','Linewidth', 0.3)
% hold on
% plot([1.4 1.4],[0 1],'-' ,'color', [0.5 0.5 0.5],'Linewidth', 0.3)
% grid on
% axis([0 inf 0.2 1]);
% title('SPRING: Operability of maintenance vessels');
% legend('Simulated','Limit: cat 15','Limit: cat 25, mono 25, mono40','Location', 'east')
% ylim([meanProbunder(1,1), inf]);
% xlabel('Significant wave height [m]');
% ylabel('Probability');
% 
% figure(4)
% plot(transpose(int_Summer),transpose(meanProbunder(2,:)) ,'-', 'Linewidth', 2)
% hold on
% plot([1.2 1.2],[0 1],'k - ','Linewidth', 0.3)
% hold on
% plot([1.4 1.4],[0 1],'-' ,'color', [0.5 0.5 0.5],'Linewidth', 0.3)
% grid on
% axis([0 inf 0.2 1]);
% title('SUMMER: Operability of maintenance vessels');
% legend('Real wave height data','Limit: cat 15','Limit: cat 25, mono 25, mono40','Location', 'east')
% ylim([meanProbunder(2,1), inf]);
% xlabel('Significant wave height [m]');
% ylabel('Probability');
% 
% 
% figure(5)
% plot(transpose(int_Autumn),transpose(meanProbunder(3,:)) ,'-', 'Linewidth', 2)
% hold on
% plot([1.2 1.2],[0 1],'k - ','Linewidth', 0.3)
% hold on
% plot([1.4 1.4],[0 1],'-' ,'color', [0.5 0.5 0.5],'Linewidth', 0.3)
% grid on
% axis([0 inf 0.2 1]);
% title(' AUTUMN: Operability of maintenance vessels');
% legend('Real wave height data','Limit: cat 15','Limit: cat 25, mono 25, mono40','Location', 'east')
% ylim([meanProbunder(3,1), inf]);
% xlabel('Significant wave height [m]');
% ylabel('Probability');


% figure(6)
% plot(transpose(int_Year),transpose(meanProbunder(6,:)) ,'-', 'Linewidth', 2)
% hold on
% plot([1.2 1.2],[0 1],'k - ','Linewidth', 0.3)
% hold on
% plot([1.4 1.4],[0 1],'-' ,'color', [0.5 0.5 0.5],'Linewidth', 0.3)
% grid on
% axis([0 inf 0.2 1]);
% title('A FULL YEAR: Operability of maintenance vessels');
% legend('Real wave height data','Limit: cat 15','Limit: cat 25, mono 25, mono40','Location', 'east')
% ylim([meanProbunder(6,1), inf]);
% xlabel('Significant wave height [m]');
% ylabel('Probability');



%% Shows the real weather and  how the wave height changes
% figure(7)
% plot(Spring)
% 
% figure(8)
% plot(Summer)
% 
% figure(9)
% plot(Autumn)
% 
% figure(10)
% plot(Winter)
% 
% figure(11)
% plot(HighSeason)
% 
% figure(12)
% plot(Year)

% 
clearvars i j p y b r interval...
    int_Summer int_Autumn int_Year int_Spring int_Winter int_HighSeason...
    Max_Sp Max_Sum Max_Au Max_Wi Max_His Max_Y IntervalWinter IntervalHighS...
    Spring Summer Autumn Winter HighSeason Year popHighS popWinter count...
    Chron_Spring Chron_Summer Chron_Autumn Chron_Winter Chron_HighSeason...
    Chron_Year a1 populationHighS populationWinter
