%%%%%%%%%%%%%%%%%%%%   Running the Simulink model from matlab %%%%%%%%%%%%%%%%%%
% This script simulates the SivEvent model 100 times where a weather
% sceanario is simulated in order to check the robustness of the optmized 
% solution.
% When the simulation is complete, all data from the simulation are
% stored in .mat files. 


clear all;
k=100;   % Number of simulations
n=1250;     % hours of simimulation. Have 50 days in the simulation  
num_states=10; % Number of states in the markov chain

% Obtaing data of time windows, type of job, vessel, duration of each job,
% factor and wave limits for each vessel
Typ=xlsread('Duration_for_Simulation', 'Type', 'A1:J20');
Twind=xlsread('Duration_for_Simulation','TimeWindow','A1:A45');
Twind2=xlsread('Duration_for_Simulation','TimeWindow','B1:B45');
VesseL=xlsread('Duration_for_Simulation','Vessel','A1:A20');
Lim=xlsread('Duration_for_Simulation','Limit','A1:A5');
Work=xlsread('Duration_for_Simulation','Job','A1:F45');
factor=xlsread('Duration_for_Simulation','Factor','A1:A19');



%% % Simulation of the weather scenarios based on Markov chains 
   % Choose between the two weather scenarios. Comment out the one that are
   % not needed. 
% %High Season: Markov chain transition probability matrix
% P=[0.9149  0.0831  0.0010  0.0010       0       0       0       0       0       0;
%    0.0859  0.8251  0.0848  0.0031  0.0010       0       0       0       0       0;
%         0  0.1824  0.7021  0.1132  0.0023       0       0       0       0       0;
%    0.0054  0.0269  0.2527  0.5914  0.1183  0.0054       0       0       0       0;
%         0  0.0154  0.0154  0.3538  0.4615  0.1538       0       0       0       0;
%         0       0       0       0  0.2683  0.5122  0.2195       0       0       0;
%         0       0       0       0       0  0.5000  0.4444  0.0556       0       0;
%         0       0       0       0       0       0  0.1250  0.7500  0.1250       0;
%         0       0       0       0       0       0       0  0.3333  0.3333  0.3333;
%         0       0       0       0       0       0       0       0  0.3333  0.6667];

% Winter: Markov chain transition probability matrix     
P=[0.3333  0.5833       0       0  0.0833       0       0       0       0       0;
   0.0165  0.7783  0.1981  0.0071       0       0       0       0       0       0;
        0  0.1403  0.7063  0.1518  0.0017       0       0       0       0       0;
        0  0.0051  0.2392  0.6107  0.1374  0.0076       0       0       0       0;
   0.0035       0  0.0035  0.1938  0.6263  0.1661  0.0069       0       0       0;
        0       0       0  0.0054  0.2703  0.5459  0.1784       0       0       0;
        0       0       0       0  0.0161  0.2661  0.5645  0.1532       0       0;
        0       0       0       0       0       0  0.2184  0.6207  0.1609       0;
        0       0       0       0       0       0       0  0.4242  0.4242  0.1515;
        0       0       0       0       0       0       0       0  0.8333  0.1667];       
 

simMC=zeros(1,n); 
datk=zeros(n,k);
states=zeros(n,1);

%Making a matrix with numbers from 0 to one to use in the simulation
% to randomly choose duration of each job within the ranges

Random=rand(1100,100);

for b=1:k   
    r=rand;
    
    % SimMC(1) chooses the probability to start in each state.
    
    % only valid for High Season 
    % Retrieved from the Markov chains simulation
    simMC(1)=sum(r >= cumsum([0, 0.3587, 0.3537 ,0.1621 ,0.0699, ...
                       0.0249, 0.0162, 0.0073, 0.0039, 0.0016]))-1;  
    
    % %only valid for Winter
    % %The values are obtained from the Markov chains simulation
    simMC(1)=sum(r >= cumsum([0, 0.0057, 0.1969, 0.2821, 0.1801, ...
               0.1327, 0.0853, 0.0575, 0.0410, 0.0158, 0.0029]))-1; 

    % Simulate the weather based on the transition matrix
    for t=2:n 
        
        % Using Monte Carlo simulation to find the probability of which 
        % state that should be the next.
       simMC(t)=randsample(num_states,1,true,P(simMC(t-1)+1,:))-1 ;
    end
    
    datk(:,b)=transpose(simMC);

end 


% Generating zero matrices to reduce the simulation time
%Week 1
W1R1=zeros(20,k);
W1R2=zeros(20,k);
W1R3=zeros(20,k);
W1R4=zeros(20,k);
W1R5=zeros(20,k);
%Week 2
W2R1=zeros(20,k);
W2R2=zeros(20,k);
W2R3=zeros(20,k);
W2R4=zeros(20,k);
W2R5=zeros(20,k);
%Week 3
W3R1=zeros(20,k);
W3R2=zeros(20,k);
W3R3=zeros(20,k);
W3R4=zeros(20,k);
W3R5=zeros(20,k);
% Week 4 
W4R1=zeros(20,k);
W4R2=zeros(20,k);
W4R3=zeros(20,k);
W4R4=zeros(20,k);
W4R5=zeros(20,k);
% Finishing routes
Exit_time=zeros(20,k);


% Make two .mat files to store data to reduce the simulation time and 
% be able to clear them when they are not needed. 
DataR= 'Retrieve_sim_data';
excelR='excelR_used_for_sim';

save(DataR,'W1R1','W1R2', 'W1R3', 'W1R4', 'W1R5','W2R1','W2R2', 'W2R3',... 
    'W2R4', 'W2R5','W3R1','W3R2', 'W3R3', 'W3R4', 'W3R5','W4R1','W4R2',...
    'W4R3', 'W4R4', 'W4R5', 'Exit_time');


clearvars b count n P r t v w num_states simMC
%% Running the simulation
%Normally, the first few simulations requires more simulation time. 

for i =1:k  
    
    tic
    
    % Load the SimEvent model to avoid having to open it.
    load_system('Simulation_Model');
    
    % Chooses column i of rand variables that is needed in the model.  
    tif=Random(:,i);
    % Shows the weather states and that is needed for the model to run.
    states=datk(:,i);
    
    save(excelR,'Typ', 'Twind', 'Twind2', 'VesseL', 'Lim','Work',...
        'factor','datk','i','k','Random');
    
    % Remove all data that are not needed for the simulation.
    % Will reduce the simulation time. 
    clearvars W1R1 W1R2 W1R3 W1R4 W1R5 W2R1 W2R2 W2R3 W2R4 W2R5...
              W3R1 W3R2 W3R3 W3R4 W3R5 W4R1 W4R2 W4R3 W4R4 W4R5...
              Exit_time DataR excelR datk Random
          
    tout=zeros(1500,1);
    
    %Run the SimEvent model 
    sim('Simulation_Model');
    
    % Retrieve the data that are needed to store data from the simulation 
    DataR= 'Retrieve_sim_data';
    load(DataR);
    
 % Get the data from the SimEvent model and storing them in a matrix
 % The if sentence checks if a vessel executes the current route. If we had
 % not checked, we would get an error due to trying to store data from an
 % empty matrix.
 
    
     if  0< VesseL(1)
     % Week 1 Route 1
         W1R1(1:2,i)=W1R1J1.Time;
         W1R1(3:4,i)=W1R1J2.Time;
         W1R1(5:6,i)=W1R1J3.Time;
         W1R1(7:8,i)=W1R1J4.Time;
         W1R1(9:10,i)=W1R1J5.Time;
         W1R1(11:12,i)=W1R1J6.Time;
         W1R1(13:14,i)=W1R1J7.Time;
         W1R1(15:16,i)=W1R1J8.Time;
         W1R1(17:18,i)=W1R1J9.Time;
         W1R1(19:20,i)=W1R1J10.Time;
         Exit_time(1,i)=W1R1EX.Time;
     end

     if 0< VesseL(2)
     % Week 1 Route 2    
         W1R2(1:2,i)=W1R2J1.Time;
         W1R2(3:4,i)=W1R2J2.Time;
         W1R2(5:6,i)=W1R2J3.Time;
         W1R2(7:8,i)=W1R2J4.Time;
         W1R2(9:10,i)=W1R2J5.Time;
         W1R2(11:12,i)=W1R2J6.Time;
         W1R2(13:14,i)=W1R2J7.Time;
         W1R2(15:16,i)=W1R2J8.Time;
         W1R2(17:18,i)=W1R2J9.Time;
         W1R2(19:20,i)=W1R2J10.Time;
         Exit_time(2,i)=W1R2EX.Time;
    end

    if 0< VesseL(3)
    % Week 1 Route 3    
         W1R3(1:2,i)=W1R3J1.Time;
         W1R3(3:4,i)=W1R3J2.Time;
         W1R3(5:6,i)=W1R3J3.Time;
         W1R3(7:8,i)=W1R3J4.Time;
         W1R3(9:10,i)=W1R3J5.Time;
         W1R3(11:12,i)=W1R3J6.Time;
         W1R3(13:14,i)=W1R3J7.Time;
         W1R3(15:16,i)=W1R3J8.Time;
         W1R3(17:18,i)=W1R3J9.Time;
         W1R3(19:20,i)=W1R3J10.Time;
         Exit_time(3,i)=W1R3EX.Time;
     end

    if 0< VesseL(4)
    % Week 1 Route 4    
         W1R4(1:2,i)=W1R4J1.Time;
         W1R4(3:4,i)=W1R4J2.Time;
         W1R4(5:6,i)=W1R4J3.Time;
         W1R4(7:8,i)=W1R4J4.Time;
         W1R4(9:10,i)=W1R4J5.Time;
         W1R4(11:12,i)=W1R4J6.Time;
         W1R4(13:14,i)=W1R4J7.Time;
         W1R4(15:16,i)=W1R4J8.Time;
         W1R4(17:18,i)=W1R4J9.Time;
         W1R4(19:20,i)=W1R4J10.Time;
         Exit_time(4,i)=W1R4EX.Time;
    end

    if 0< VesseL(5)
    % Week 1 Route 5    
         W1R5(1:2,i)=W1R5J1.Time;
         W1R5(3:4,i)=W1R5J2.Time;
         W1R5(5:6,i)=W1R5J3.Time;
         W1R5(7:8,i)=W1R5J4.Time;
         W1R5(9:10,i)=W1R5J5.Time;
         W1R5(11:12,i)=W1R5J6.Time;
         W1R5(13:14,i)=W1R5J7.Time;
         W1R5(15:16,i)=W1R5J8.Time;
         W1R5(17:18,i)=W1R5J9.Time;
         W1R5(19:20,i)=W1R5J10.Time;
         Exit_time(5,i)=W1R5EX.Time;    
    end
    if  0< VesseL(6)
     % Week 2 Route 1
         W2R1(1:2,i)=W2R1J1.Time;
         W2R1(3:4,i)=W2R1J2.Time;
         W2R1(5:6,i)=W2R1J3.Time;
         W2R1(7:8,i)=W2R1J4.Time;
         W2R1(9:10,i)=W2R1J5.Time;
         W2R1(11:12,i)=W2R1J6.Time;
         W2R1(13:14,i)=W2R1J7.Time;
         W2R1(15:16,i)=W2R1J8.Time;
         W2R1(17:18,i)=W2R1J9.Time;
         W2R1(19:20,i)=W2R1J10.Time;
         Exit_time(6,i)=W2R1EX.Time;
    end
    if 0< VesseL(7)
     % Week 2 Route 2    
         W2R2(1:2,i)=W2R2J1.Time;
         W2R2(3:4,i)=W2R2J2.Time;
         W2R2(5:6,i)=W2R2J3.Time;
         W2R2(7:8,i)=W2R2J4.Time;
         W2R2(9:10,i)=W2R2J5.Time;
         W2R2(11:12,i)=W2R2J6.Time;
         W2R2(13:14,i)=W2R2J7.Time;
         W2R2(15:16,i)=W2R2J8.Time;
         W2R2(17:18,i)=W2R2J9.Time;
         W2R2(19:20,i)=W2R2J10.Time;
         Exit_time(7,i)=W2R2EX.Time;
    end
    if 0< VesseL(8)
    % Week 2 Route 3    
         W2R3(1:2,i)=W2R3J1.Time;
         W2R3(3:4,i)=W2R3J2.Time;
         W2R3(5:6,i)=W2R3J3.Time;
         W2R3(7:8,i)=W2R3J4.Time;
         W2R3(9:10,i)=W2R3J5.Time;
         W2R3(11:12,i)=W2R3J6.Time;
         W2R3(13:14,i)=W2R3J7.Time;
         W2R3(15:16,i)=W2R3J8.Time;
         W2R3(17:18,i)=W2R3J9.Time;
         W2R3(19:20,i)=W2R3J10.Time;
         Exit_time(8,i)=W2R3EX.Time;
    end
    if 0< VesseL(9)
    % Week 2 Route 4    
         W2R4(1:2,i)=W2R4J1.Time;
         W2R4(3:4,i)=W2R4J2.Time;
         W2R4(5:6,i)=W2R4J3.Time;
         W2R4(7:8,i)=W2R4J4.Time;
         W2R4(9:10,i)=W2R4J5.Time;
         W2R4(11:12,i)=W2R4J6.Time;
         W2R4(13:14,i)=W2R4J7.Time;
         W2R4(15:16,i)=W2R4J8.Time;
         W2R4(17:18,i)=W2R4J9.Time;
         W2R4(19:20,i)=W2R4J10.Time;
         Exit_time(9,i)=W2R4EX.Time;
    end

    if 0< VesseL(10)
    % Week 2 Route 5    
         W2R5(1:2,i)=W2R5J1.Time;
         W2R5(3:4,i)=W2R5J2.Time;
         W2R5(5:6,i)=W2R5J3.Time;
         W2R5(7:8,i)=W2R5J4.Time;
         W2R5(9:10,i)=W2R5J5.Time;
         W2R5(11:12,i)=W2R5J6.Time;
         W2R5(13:14,i)=W2R5J7.Time;
         W2R5(15:16,i)=W2R5J8.Time;
         W2R5(17:18,i)=W2R5J9.Time;
         W2R5(19:20,i)=W2R5J10.Time;
         Exit_time(10,i)=W2R5EX.Time;    
    end

    if  0< VesseL(11)
     % Week 3 Route 1
         W3R1(1:2,i)=W3R1J1.Time;
         W3R1(3:4,i)=W3R1J2.Time;
         W3R1(5:6,i)=W3R1J3.Time;
         W3R1(7:8,i)=W3R1J4.Time;
         W3R1(9:10,i)=W3R1J5.Time;
         W3R1(11:12,i)=W3R1J6.Time;
         W3R1(13:14,i)=W3R1J7.Time;
         W3R1(15:16,i)=W3R1J8.Time;
         W3R1(17:18,i)=W3R1J9.Time;
         W3R1(19:20,i)=W3R1J10.Time;
         Exit_time(11,i)=W3R1EX.Time;
    end
    if 0< VesseL(12)
     % Week 3 Route 2    
         W3R2(1:2,i)=W3R2J1.Time;
         W3R2(3:4,i)=W3R2J2.Time;
         W3R2(5:6,i)=W3R2J3.Time;
         W3R2(7:8,i)=W3R2J4.Time;
         W3R2(9:10,i)=W3R2J5.Time;
         W3R2(11:12,i)=W3R2J6.Time;
         W3R2(13:14,i)=W3R2J7.Time;
         W3R2(15:16,i)=W3R2J8.Time;
         W3R2(17:18,i)=W3R2J9.Time;
         W3R2(19:20,i)=W3R2J10.Time;
         Exit_time(12,i)=W3R2EX.Time;
    end
    if 0< VesseL(13)
    % Week 3 Route 3    
         W3R3(1:2,i)=W3R3J1.Time;
         W3R3(3:4,i)=W3R3J2.Time;
         W3R3(5:6,i)=W3R3J3.Time;
         W3R3(7:8,i)=W3R3J4.Time;
         W3R3(9:10,i)=W3R3J5.Time;
         W3R3(11:12,i)=W3R3J6.Time;
         W3R3(13:14,i)=W3R3J7.Time;
         W3R3(15:16,i)=W3R3J8.Time;
         W3R3(17:18,i)=W3R3J9.Time;
         W3R3(19:20,i)=W3R3J10.Time;
         Exit_time(13,i)=W3R3EX.Time;
    end
    if 0< VesseL(14)
    % Week 3 Route 4    
         W3R4(1:2,i)=W3R4J1.Time;
         W3R4(3:4,i)=W3R4J2.Time;
         W3R4(5:6,i)=W3R4J3.Time;
         W3R4(7:8,i)=W3R4J4.Time;
         W3R4(9:10,i)=W3R4J5.Time;
         W3R4(11:12,i)=W3R4J6.Time;
         W3R4(13:14,i)=W3R4J7.Time;
         W3R4(15:16,i)=W3R4J8.Time;
         W3R4(17:18,i)=W3R4J9.Time;
         W3R4(19:20,i)=W3R4J10.Time;
         Exit_time(14,i)=W3R4EX.Time;
    end

    if 0< VesseL(15)
    % Week 3 Route 5    
         W3R5(1:2,i)=W3R5J1.Time;
         W3R5(3:4,i)=W3R5J2.Time;
         W3R5(5:6,i)=W3R5J3.Time;
         W3R5(7:8,i)=W3R5J4.Time;
         W3R5(9:10,i)=W3R5J5.Time;
         W3R5(11:12,i)=W3R5J6.Time;
         W3R5(13:14,i)=W3R5J7.Time;
         W3R5(15:16,i)=W3R5J8.Time;
         W3R5(17:18,i)=W3R5J9.Time;
         W3R5(19:20,i)=W3R5J10.Time;
         Exit_time(15,i)=W3R5EX.Time;    
    end

    if  0< VesseL(16)
     % Week 4 Route 1
         W4R1(1:2,i)=W4R1J1.Time;
         W4R1(3:4,i)=W4R1J2.Time;
         W4R1(5:6,i)=W4R1J3.Time;
         W4R1(7:8,i)=W4R1J4.Time;
         W4R1(9:10,i)=W4R1J5.Time;
         W4R1(11:12,i)=W4R1J6.Time;
         W4R1(13:14,i)=W4R1J7.Time;
         W4R1(15:16,i)=W4R1J8.Time;
         W4R1(17:18,i)=W4R1J9.Time;
         W4R1(19:20,i)=W4R1J10.Time;
         Exit_time(16,i)=W4R1EX.Time;
    end
    if 0< VesseL(17)
     % Week 4 Route 2    
         W4R2(1:2,i)=W4R2J1.Time;
         W4R2(3:4,i)=W4R2J2.Time;
         W4R2(5:6,i)=W4R2J3.Time;
         W4R2(7:8,i)=W4R2J4.Time;
         W4R2(9:10,i)=W4R2J5.Time;
         W4R2(11:12,i)=W4R2J6.Time;
         W4R2(13:14,i)=W4R2J7.Time;
         W4R2(15:16,i)=W4R2J8.Time;
         W4R2(17:18,i)=W4R2J9.Time;
         W4R2(19:20,i)=W4R2J10.Time;
         Exit_time(17,i)=W4R2EX.Time;
    end
    if 0< VesseL(18)
    % Week 4 Route 3    
         W4R3(1:2,i)=W4R3J1.Time;
         W4R3(3:4,i)=W4R3J2.Time;
         W4R3(5:6,i)=W4R3J3.Time;
         W4R3(7:8,i)=W4R3J4.Time;
         W4R3(9:10,i)=W4R3J5.Time;
         W4R3(11:12,i)=W4R3J6.Time;
         W4R3(13:14,i)=W4R3J7.Time;
         W4R3(15:16,i)=W4R3J8.Time;
         W4R3(17:18,i)=W4R3J9.Time;
         W4R3(19:20,i)=W4R3J10.Time;
         Exit_time(18,i)=W4R3EX.Time;
    end
    if 0< VesseL(19)
    % Week 4 Route 4    
         W4R4(1:2,i)=W4R4J1.Time;
         W4R4(3:4,i)=W4R4J2.Time;
         W4R4(5:6,i)=W4R4J3.Time;
         W4R4(7:8,i)=W4R4J4.Time;
         W4R4(9:10,i)=W4R4J5.Time;
         W4R4(11:12,i)=W4R4J6.Time;
         W4R4(13:14,i)=W4R4J7.Time;
         W4R4(15:16,i)=W4R4J8.Time;
         W4R4(17:18,i)=W4R4J9.Time;
         W4R4(19:20,i)=W4R4J10.Time;
         Exit_time(19,i)=W4R4EX.Time;
    end

    if 0< VesseL(20)
    % Week 4 Route 5    
         W4R5(1:2,i)=W4R5J1.Time;
         W4R5(3:4,i)=W4R5J2.Time;
         W4R5(5:6,i)=W4R5J3.Time;
         W4R5(7:8,i)=W4R5J4.Time;
         W4R5(9:10,i)=W4R5J5.Time;
         W4R5(11:12,i)=W4R5J6.Time;
         W4R5(13:14,i)=W4R5J7.Time;
         W4R5(15:16,i)=W4R5J8.Time;
         W4R5(17:18,i)=W4R5J9.Time;
         W4R5(19:20,i)=W4R5J10.Time;
         Exit_time(20,i)=W4R5EX.Time;    
    end
    
   % Save the new obtained data from the simulation. 
   DataR= 'Retrieve_sim_data';
   save(DataR,'W1R1','W1R2', 'W1R3', 'W1R4', 'W1R5','W2R1','W2R2',...
       'W2R3','W2R4', 'W2R5','W3R1','W3R2', 'W3R3', 'W3R4', 'W3R5',...
       'W4R1','W4R2','W4R3', 'W4R4', 'W4R5', 'Exit_time'); 
    
    % Not necessary, but interesting to see the progress of the 
    % simulation and how much time each simulation takes.
    Tid=i
    toc
    
    % Clear all variables to remove all unnecessary global variables 
    % that are increasing the simulation time. 
    clear all;

    % Retrieve the variables that are needed for the simulation and
    % for loop to proceed the simulation
    excelR='excelR_used_for_sim';
    load(excelR);
end
clear all; 