%Script for creating markov chain for significant waveheight for both
%salmar and marine harvest data

%clear all;


numStates = 10;
HsMH = xlsread('Hs data Marine Harvest and Salmar.xlsx','Sheet1','D7453:D9612');
HsSalmar = xlsread('Hs data Marine Harvest and Salmar.xlsx','Sheet1','H3:H9259');
% Winter = 7453 ->  9612
% Week 26-41 = 3685 -> 6372
u1 = max(HsMH);
u2 = max(HsSalmar);

for a = 1:2
    if a == 1
    stateRange1 = u1 / numStates;
    % State values - stateRange, 2xstateRange and so on up til ul
    stateValues1 = stateRange1:stateRange1:u1;
    % Initialize 1D-matrix holding the state of each data point
    HsState1 = zeros(length(HsMH),1);

    % Find each data points state
    for i = 1:length(HsMH)
    % For each data point
    for j = 1:numStates
        % For each state
        if HsMH(i) <= stateValues1(j)
            % Data point is in state j
            HsState1(i) = j;
            % This data point is categorized, so we break and move to the
            % next data point
            break;
        end
    end
    end

% Find transitions
transitions1 = zeros(numStates);
for t = 1:length(HsState1)-1
    % HsState(t) represents the state and HsState(t+1) represents the state
    % it transitions to
    transitions1(HsState1(t),HsState1(t+1)) = transitions1(HsState1(t),HsState1(t+1)) + 1;
end

P1 = transitions1;
% Normalize each row in the transition matrix so each row sums to 1
for i = 1:numStates
    P1(i,:) = P1(i,:) / sum( P1(i,:) );
end

% Check to see if there are any absorbing states
% i.e. P(i,j) == 1 where i=j
absorbstate1 = zeros(numStates);
for i = 1:numStates
    for j = 1:numStates
        if P1(i,j) == 1
            absorbstate1(i,j) = absorbstate1(i,j) + 1;
        end
    end
end
if sum(sum(absorbstate1)) >= 1
    error('Absorbing states. Stopping. Consider reducing number of states or check data.');
end

%% Transition matrix is now ready in P

% How many state transitions to perform
% Lower this number to show how fewer replications affects results
% for example, 100, 1000, length(Hs), 10000
numReplications = 10000000;

% Random number seed
rng(5);

% Set starting state - should sample randomly
state1 = randi(numStates);

states1 = zeros(numReplications,1);

for i = 1:numReplications
	% Sample a new random value in range [0,1]
    r = rand();
    
    for j = 1:numStates
        prob = 0;
        % Accumulate probabilities
        for k = 1:j
            prob = prob + P1(state1,k);
        end
        
        if r <= prob
            % New state is found, j
            state1 = j;
            
            % Store the state we transition to
            states1(i) = j;

            % Break ends the current for loop, and returns to the outer
            % loop, which will sample a new random value and start over
            break;
        end
    end
end

% If needed, Hs can be compared directly to the simulated results
simValues1 = zeros(numReplications,1);
for i = 1:numReplications
    simValues1(i) = (states1(i) * stateRange1) - stateRange1/2;
end


% Plot the distribution for the original data points and the simulated sea
% states. The number of samples won't correlate, but the general shape
% should correlate somewhat.
% figure(1);
% hist(HsMH,numStates);
% title('Data points HsMH');
% figure(2);
% hist(simValues1,numStates);
% title('Simulation results HsMH');

    elseif a == 2
    stateRange2 = u2 / numStates;
    % State values - stateRange, 2xstateRange and so on up til ul
    stateValues2 = stateRange2:stateRange2:u2;
    % Initialize 1D-matrix holding the state of each data point
    HsState2 = zeros(length(HsSalmar),1);

    % Find each data points state
    for i = 1:length(HsSalmar)
    % For each data point
    for j = 1:numStates
        % For each state
        if HsSalmar(i) <= stateValues2(j)
            % Data point is in state j
            HsState2(i) = j;
            % This data point is categorized, so we break and move to the
            % next data point
            break;
        end
    end
    end

% Find transitions
transitions2 = zeros(numStates);
for t = 1:length(HsState2)-1
    % HsState(t) represents the state and HsState(t+1) represents the state
    % it transitions to
    transitions2(HsState2(t),HsState2(t+1)) = transitions2(HsState2(t),HsState2(t+1)) + 1;
end

P2 = transitions2;
% Normalize each row in the transition matrix so each row sums to 1
for i = 1:numStates
    P2(i,:) = P2(i,:) / sum( P2(i,:) );
end

% Check to see if there are any absorbing states
% i.e. P(i,j) == 1 where i=j
absorbstate2 = zeros(numStates);
for i = 1:numStates
    for j = 1:numStates
        if P2(i,j) == 1
            absorbstate2(i,j) = absorbstate2(i,j) + 1;
        end
    end
end
if sum(sum(absorbstate2)) >= 1
    error('Absorbing states. Stopping. Consider reducing number of states or check data.');
end

%% Transition matrix is now ready in P

% How many state transitions to perform
% Lower this number to show how fewer replications affects results
% for example, 100, 1000, length(Hs), 10000
numReplications = 10000000;

% Random number seed
rng(1234);

% Set starting state - should sample randomly
state2 = randi(numStates);

states2 = zeros(numReplications,1);

for i = 1:numReplications
	% Sample a new random value in range [0,1]
    r = rand();
    
    for j = 1:numStates
        prob = 0;
        % Accumulate probabilities
        for k = 1:j
            prob = prob + P2(state2,k);
        end
        
        if r <= prob
            % New state is found, j
            state2 = j;
            
            % Store the state we transition to
            states2(i) = j;

            % Break ends the current for loop, and returns to the outer
            % loop, which will sample a new random value and start over
            break;
        end
    end
end

% If needed, Hs can be compared directly to the simulated results
simValues2 = zeros(numReplications,1);
for i = 1:numReplications
    simValues2(i) = (states2(i) * stateRange2) - stateRange2/2;
end


% Plot the distribution for the original data points and the simulated sea
% states. The number of samples won't correlate, but the general shape
% should correlate somewhat.
% figure(3);
% hist(HsSalmar,numStates);
% title('Data points HsSalmar');
% figure(4);
% hist(simValues2,numStates);
% title('Simulation results HsSalmar');
    end
end
